package com.persagy.account.controller;

import java.util.Date;
import java.util.List;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.google.common.collect.Lists;
import com.persagy.account.pojo.dto.SaasProduct;
import com.persagy.account.pojo.dto.SaasProject;
import com.persagy.account.pojo.dto.SaasProjectProduct;
import com.persagy.account.pojo.vo.product.SaasProjectProductBatchVO;
import com.persagy.account.pojo.vo.product.SaasProjectProductCreateVO;
import com.persagy.account.pojo.vo.product.SaasProjectProductQueryVO;
import com.persagy.account.service.ISaasProductService;
import com.persagy.account.service.ISaasProjectProductService;
import com.persagy.account.service.ISaasProjectService;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseResultUtil;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 项目签约产品
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-05-07 12:02:40
 */

@Api(tags = "产品项目签约管理")
@RestController
@RequestMapping(value = "/projectProduct", method = RequestMethod.POST)
public class SaasProjectProductController {
	
    @Autowired
    private ISaasProjectService saasProjectService;
    
    @Autowired
    private ISaasProductService saasProductService;
    
    @Autowired
    private ISaasProjectProductService saasProjectProductService;
    
    /**
     * 查询产品项目签约信息
     */
    @ApiOperation(value = "列表查询")
    @RequestMapping(value = "querySaasProjectProductList")
    public ResponseResult querySaasProjectProductList(@RequestBody @Validated SaasProjectProductQueryVO model) {
    	List<SaasProjectProduct> list = null;
    	if (model.isShowProjectName()) {
    		list = this.saasProjectProductService.querySaasProjectProductList(model.getContractId(), model.getGroupCode(), model.getProjectId(), model.getProductId());
		} else {
			LambdaQueryWrapper<SaasProjectProduct> queryWrapper = new SaasProjectProduct.BuilderQueryWrapper().contractIdEq(model.getContractId())
	    			.groupCodeEq(model.getGroupCode()).projectIdEq(model.getProjectId()).productIdEq(model.getProductId()).builder();
	    	list = this.saasProjectProductService.list(queryWrapper);
		}
    	if (CollectionUtil.isEmpty(list)) {
			return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
		}
        return ResponseResultUtil.successResult(list, (long)list.size());
    }

    /**
     * 新增产品项目签约信息
     */
	@ApiOperation(value = "保存")
	@RequestMapping(value = "createSaasProjectProduct")
    public ResponseResult createSaasProjectProduct(@RequestBody @Validated SaasProjectProductCreateVO model) {
		// 验证项目是否存在
		SaasProject saasProject = this.saasProjectService.getOne(model.getGroupCode(), model.getProjectId());
		if (saasProject == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "不存在此项目");
		}
		// 验证产品是否存在
		SaasProduct saasProduct = this.saasProductService.getById(model.getProductId());
		if (saasProduct == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "不存在此产品");
		}
		SaasProjectProduct saasProjectProduct = new SaasProjectProduct();
		BeanUtils.copyProperties(model, saasProjectProduct);
		saasProjectProduct.setProductContract(new Date());
        boolean result = this.saasProjectProductService.save(saasProjectProduct);
        return result ? ResponseResultUtil.successResult("保存成功") : ResponseResultUtil.errorResult("保存失败");
    }
	
    /**
     * 批量更新产品项目签约信息，删除新增
     */
	@ApiOperation(value = "替换")
	@RequestMapping(value = "replaceSaasProjectProduct")
    public ResponseResult replaceSaasProjectProduct(@RequestBody @Validated SaasProjectProductBatchVO model) {
		// 验证产品是否存在
		SaasProduct saasProduct = this.saasProductService.getById(model.getProductId());
		if (saasProduct == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "不存在此产品");
		}
        boolean result = this.saasProjectProductService.replaceSaasProjectProduct(model.getContractId(), model.getGroupCode(), model.getProductId(), model.getProjectIds());
        return result ? ResponseResultUtil.successResult("重置成功") : ResponseResultUtil.errorResult("重置失败");
    }
    
    /**
     * 更新产品项目签约信息
     */
	@ApiOperation(value = "更新")
	@RequestMapping(value = "updateSaasProjectProduct")
    public ResponseResult updateSaasProjectProduct(@RequestBody @Validated SaasProjectProductCreateVO model) {
		// 验证项目是否存在
		SaasProject saasProject = this.saasProjectService.getOne(model.getGroupCode(), model.getProjectId());
		if (saasProject == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "不存在此项目");
		}
		// 验证产品是否存在
		SaasProduct saasProduct = this.saasProductService.getById(model.getProductId());
		if (saasProduct == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "不存在此产品");
		}
		SaasProjectProduct saasProjectProduct = new SaasProjectProduct();
		BeanUtils.copyProperties(model, saasProjectProduct);
		LambdaUpdateWrapper<SaasProjectProduct> updateWrapper = new SaasProjectProduct.BuilderUpdateWrapper().contractIdEq(model.getContractId())
				.groupCodeEq(model.getGroupCode()).productIdEq(model.getProductId()).builder();
        boolean result = this.saasProjectProductService.update(saasProjectProduct, updateWrapper);
        return result ? ResponseResultUtil.successResult("更新成功") : ResponseResultUtil.errorResult("更新失败");
    }

    /**
     * 物理删除产品项目签约信息
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "deleteSaasProjectProduct")
    public ResponseResult deleteSaasProjectProduct(@RequestBody @Validated SaasProjectProductQueryVO model) {
    	LambdaQueryWrapper<SaasProjectProduct> queryWrapper = new SaasProjectProduct.BuilderQueryWrapper().contractIdEq(model.getContractId())
    			.groupCodeEq(model.getGroupCode()).projectIdEq(model.getProjectId()).productIdEq(model.getProductId()).builder();
        boolean result = this.saasProjectProductService.remove(queryWrapper);
        return result ? ResponseResultUtil.successResult("删除成功") : ResponseResultUtil.errorResult("删除失败");
    }
    
}
