package com.persagy.account.controller;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.persagy.account.manage.SaasRoleMenuHandler;
import com.persagy.account.pojo.dto.SaasGroup;
import com.persagy.account.pojo.dto.SaasRole;
import com.persagy.account.pojo.vo.role.SaasRoleCreateVO;
import com.persagy.account.pojo.vo.role.SaasRoleDeleteVO;
import com.persagy.account.pojo.vo.role.SaasRolePageVO;
import com.persagy.account.pojo.vo.role.SaasRoleQueryByNameVO;
import com.persagy.account.pojo.vo.role.SaasRoleQueryVO;
import com.persagy.account.service.ISaasAccountRoleService;
import com.persagy.account.service.ISaasGroupService;
import com.persagy.account.service.ISaasRoleMenuService;
import com.persagy.account.service.ISaasRoleService;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseResultUtil;
import com.persagy.common.utils.StringUtil;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 角色信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@RestController
@Api(tags = "角色信息")
@SuppressWarnings("unchecked")
@RequestMapping(value = "/role", method = RequestMethod.POST)
public class SaasRoleController {
	
    @Autowired
    private ISaasRoleService saasRoleService;
    
    @Autowired
    private ISaasGroupService saasGroupService;
    
    @Autowired
    private SaasRoleMenuHandler saasRoleMenuHandler;
    
    @Autowired
    private ISaasRoleMenuService saasRoleMenuService;
    
    @Autowired
    private ISaasAccountRoleService saasAccountRoleService;
    
    /**
     * 查询角色信息
     */
	@ApiOperation(value = "查询角色信息")
    @RequestMapping(value = "querySaasRoleList")
    public ResponseResult querySaasRoleList(@RequestBody @Validated SaasRoleQueryVO queryVO) {
    	LambdaQueryWrapper<SaasRole> queryWrapper = new SaasRole.Builder().createQueryWrapper().idEq(queryVO.getId())
    			.idIn(queryVO.getRoleIds()).roleCodeEq(queryVO.getRoleCode()).roleNameLike(queryVO.getRoleName())
    			.groupCodeEq(queryVO.getGroupCode()).roleTypeEq(queryVO.getRoleType()).roleTypeIn(queryVO.getRoleTypes())
    			.updateUserEq(queryVO.getUpdateUser()).validEq(queryVO.getValid()).builderQueryWrapper();
    	queryWrapper.orderByDesc(SaasRole::getUpdateTime, SaasRole::getId);		// 默认更新时间降序
    	List<SaasRole> list = this.saasRoleService.list(queryWrapper);
    	if (CollectionUtil.isEmpty(list)) {
			return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
		}
    	// 额外信息补充
    	if (queryVO.isCountMenu() || queryVO.isCountFunction()) {
			for (SaasRole saasRole : list) {
				if (queryVO.isCountFunction()) {
					saasRole.setCountFunction(this.saasRoleMenuService.countFunction(saasRole.getId(), null));
				}
				if (queryVO.isCountMenu()) {
					saasRole.setCountMenu(this.saasRoleMenuService.countMenu(saasRole.getId()));
				}
			}
		}
        return ResponseResultUtil.successResult(list, (long)list.size());
    }
    
    /**
     * 分页查询角色信息
     */
    @ApiOperation(value = "分页查询角色信息")
    @RequestMapping(value = "queryRolePageList")
    public ResponseResult queryRolePageList(@RequestBody @Validated SaasRolePageVO pageVO) {
    	Page<SaasRole> page = new Page<SaasRole>(pageVO.getPage(), pageVO.getSize());
    	LambdaQueryWrapper<SaasRole> queryWrapper = new SaasRole.Builder().createQueryWrapper().idEq(pageVO.getId())
    			.roleCodeEq(pageVO.getRoleCode()).roleNameLike(pageVO.getRoleName()).roleTypeEq(pageVO.getRoleType())
    			.roleTypeIn(pageVO.getRoleTypes()).groupCodeEq(pageVO.getGroupCode()).updateUserEq(pageVO.getUpdateUser())
    			.validEq(pageVO.getValid()).builderQueryWrapper();
		queryWrapper.orderByDesc(SaasRole::getUpdateTime, SaasRole::getId);		// 默认更新时间降序
    	Page<SaasRole> pageList = this.saasRoleService.page(page, queryWrapper);
    	if (pageList == null || CollectionUtil.isEmpty(pageList.getRecords())) {
    		return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
		}
    	// 额外信息补充
    	List<SaasRole> records = pageList.getRecords();
    	// 额外信息补充
    	List<SaasRole> temp = new ArrayList<SaasRole>(records.size());
    	if (pageVO.isCountMenu() || pageVO.isCountFunction() || (pageVO.isAdminOrder() && pageVO.getPage() == 1)) {
    		for (int i = records.size() - 1; i >= 0; i--) {
    			SaasRole saasRole = records.get(i);
				if (pageVO.isCountFunction()) {
					saasRole.setCountFunction(this.saasRoleMenuService.countFunction(saasRole.getId(), null));
				}
				if (pageVO.isCountMenu()) {
					saasRole.setCountMenu(this.saasRoleMenuService.countMenu(saasRole.getId()));
				}
				// 判断是否要进行业务超管放在第一位
		    	if (pageVO.isAdminOrder() && pageVO.getPage() == 1) {
		    		temp.add(saasRole);
		    		records.remove(i);
				}
			}
		}
    	if (pageVO.isAdminOrder() && pageVO.getPage() == 1) {
    		for (SaasRole saasRole : records) {
    			temp.add(saasRole);
			}
    		return ResponseResultUtil.successResult(temp, pageList.getTotal()); 
    	}
    	return ResponseResultUtil.successResult(records, pageList.getTotal());
    }

    /**
     * 根据角色名称查询角色信息
     */
    @ApiOperation(value = "根据角色名称查询角色信息")
    @RequestMapping(value = "querySaasRoleByName")
    public ResponseResult querySaasRoleByName(@RequestBody @Validated SaasRoleQueryByNameVO queryVO) {
        SaasRole saasRole = new SaasRole();
        BeanUtils.copyProperties(queryVO,saasRole);
        LambdaQueryWrapper<SaasRole> queryWrapper = new SaasRole.Builder().createQueryWrapper().groupCodeEq(saasRole.getGroupCode())
        		.roleNameEq(saasRole.getRoleName()).roleTypeEq(saasRole.getRoleType()).builderQueryWrapper();
        List<SaasRole> list = this.saasRoleService.list(queryWrapper);
        if (CollectionUtil.isEmpty(list)) {
            return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
        }
        return ResponseResultUtil.successResult(list, (long)list.size());
    }
    
    /**
     * 新增 角色名称唯一
     */
	@ApiOperation(value = "保存")
	@RequestMapping(value = "createSaasRole")
    public ResponseResult createSaasRole(@RequestBody @Validated SaasRoleCreateVO createVO) {
		if (!SaasCommonConstant.STR_STATUS_0.equals(createVO.getRoleType())) {
			// 非系统角色，一定存在集团编码
			if (StringUtil.isBlank(createVO.getGroupCode())) {
				return ResponseResultUtil.errorResult(ResponseCode.C0341.getCode(), "集团编码必须存在");
			}
			// 验证集团信息是否存在
			SaasGroup saasGroup = this.saasGroupService.getOne(createVO.getGroupCode());
			if (saasGroup == null) {
				return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "集团信息不存在");
			}
		}
		// 角色名称集团下唯一
		boolean result = this.saasRoleService.validRoleName(createVO.getGroupCode(), createVO.getRoleName(), createVO.getRoleType(), null);
		if (!result) {
			return ResponseResultUtil.errorResult(ResponseCode.C0341.getCode(), "名称已存在");
		}
        SaasRole saasRole = new SaasRole();
        BeanUtils.copyProperties(createVO, saasRole);
        saasRole.setId(null);
        saasRole.setUpdateUser(createVO.getAccountId());
        
        this.saasRoleMenuHandler.validRoleMenus(createVO.getAuths());
        result = this.saasRoleMenuHandler.createSaasRole(saasRole, createVO.getAuths());
        return result ? ResponseResultUtil.successResult("保存成功", saasRole.getId()) : ResponseResultUtil.errorResult("保存失败");
    }
    
    /**
     * 更新
     */
	@ApiOperation(value = "更新")
	@RequestMapping(value = "updateSaasRole")
    public ResponseResult updateSaasRole(@RequestBody @Validated SaasRoleCreateVO createVO) {
		if (StringUtil.isBlank(createVO.getId())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "ID不可为空");
		}
		if (!SaasCommonConstant.STR_STATUS_0.equals(createVO.getRoleType())) {
			// 非系统角色，一定存在集团编码
			if (StringUtil.isBlank(createVO.getGroupCode())) {
				return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "集团编码必须存在");
			}
			// 验证集团信息是否存在
			SaasGroup saasGroup = this.saasGroupService.getOne(createVO.getGroupCode());
			if (saasGroup == null) {
				return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "集团信息不存在");
			}
		}
		// 角色名称集团下唯一
		boolean result = this.saasRoleService.validRoleName(createVO.getGroupCode(), createVO.getRoleName(), createVO.getRoleType(), createVO.getId());
		if (!result) {
			return ResponseResultUtil.errorResult(ResponseCode.C0341.getCode(), "名称已存在");
		}
		// 验证数据是否存在
		SaasRole role = this.saasRoleService.getOne(createVO.getId());
		if (role == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "数据不存在");
		}
		SaasRole saasRole = new SaasRole();
        BeanUtils.copyProperties(createVO, saasRole);
        saasRole.setUpdateUser(createVO.getAccountId());
        
        this.saasRoleMenuHandler.validRoleMenus(createVO.getAuths());
        result = this.saasRoleMenuHandler.updateSaasRole(saasRole, createVO.getAuths());
        return result ? ResponseResultUtil.successResult("更新成功") : ResponseResultUtil.errorResult("更新失败");
    }

    /**
     * 物理删除，验证是否允许被删除，即该角色不被任何账号使用，且非业务超管角色
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "deleteSaasRole")
    public ResponseResult deleteSaasRole(@RequestBody @Validated SaasRoleDeleteVO deleteVO) {
    	// 验证是否允许被删除
    	boolean result = this.saasAccountRoleService.isAllowDeleteRole(deleteVO.getGroupCode(), deleteVO.getId());
    	if (!result) {
    		return ResponseResultUtil.errorResult(ResponseCode.A0402.getCode(), "该角色已被使用，不允许被删除");
		}
    	result = this.saasRoleMenuHandler.deleteSaasRole(deleteVO);
        return result ? ResponseResultUtil.successResult("删除成功") : ResponseResultUtil.errorResult("删除失败");
    }
    
}
