package com.persagy.account.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.persagy.account.pojo.dto.SaasProject;

/**
 * 项目信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Mapper
public interface SaasProjectMapper extends BaseMapper<SaasProject> {

	/**
	 * 项目列表查询
	 * 
	 * @param saasProject
	 * @param startRow
	 * @param size
	 * @return
	 */
	List<SaasProject> querySaasProjectList(@Param("project") SaasProject saasProject, @Param("startRow") Integer startRow, @Param("size") Integer size);
	
	/**
	 * 分页查询时，获取总记录数
	 * 
	 * @param saasProject
	 * @return
	 */
	Integer queryCountMoreProject(@Param("project") SaasProject saasProject);
		
	/**
	 * 获取项目集合信息，关联查询
	 * 
	 * @param saasProject
	 * @param page
	 * @param size
	 * @return
	 */
	List<SaasProject> queryMoreProjectInfo(@Param("project") SaasProject saasProject, @Param("startRow") Integer startRow, @Param("size") Integer size);

	/**
	 * 查询该账户可允许访问的集团项目，权限表关联查询，默认项目名称正序排序
	 * 
	 * @param groupCode
	 * @param accountId
	 * @param valid 1-可用，0-不可用
	 * @return
	 */
	List<SaasProject> queryAllowGroupProjectList(@Param("groupCode") String groupCode, @Param("accountId") String accountId, 
			@Param("areaId") String areaId, @Param("valid") Integer valid);
	
	/**
	 * 批量插入项目信息
	 * 
	 * @param list
	 * @return
	 */
	Integer batchInsertProjectData(@Param("projects") List<SaasProject> projects);
	
	/**
	 * 批量更新项目所属区域
	 * 
	 * @param saasProject
	 * @param projectIds
	 * @return
	 */
	Integer batchUpdateProjectArea(@Param("project") SaasProject saasProject, @Param("projectIds") List<String> projectIds);

	/**
	 * 查询出能够被访问的项目总数
	 * 
	 * @param groupCode 集团编码
	 * @param accountId 账号ID
	 * @param areaId 底层区域ID
	 * @return
	 */
	Integer countAllowProjectNum(@Param("groupCode") String groupCode, @Param("accountId") String accountId, 
			@Param("areaId") String areaId, @Param("valid") Integer valid);
	
}
