package com.persagy.account.pojo.dto;

import java.util.Date;
import java.util.List;
import java.util.Set;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler;
import com.persagy.common.annotation.IdGeneratorPrefix;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.model.BaseEntity;
import com.persagy.common.utils.StringUtil;

import cn.hutool.core.collection.CollectionUtil;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

/**
 * 账号信息
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Getter
@Setter
@ToString
@EqualsAndHashCode(callSuper = false)
@TableName(value = "saas_account", autoResultMap = true)
@IdGeneratorPrefix(SaasCommonConstant.SAAS_ACCOUNT_ID_PREFIX)
public class SaasAccount extends BaseEntity<SaasAccount> {
	private static final long serialVersionUID = 3222916987593249671L;

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String groupCode; // 集团编码

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY, typeHandler = FastjsonTypeHandler.class)
	private Set<String> terminal; // 应用ID， PC-电脑端，APP-手机端
	
	@TableField(exist = false)
	private String appId; 	// 应用ID，返回给前端的，也即时登录时，指定的

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String username; // 登录用户名

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String password; // 登录密码

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String phoneNum; // 手机号

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String mail; // 邮箱

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String headPortrait; // 头像，图片URL

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String accountType;		//账号类型, 0-超级管理员(所有集团)，1-单集团管理员(单集团)，2-普通账号

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String accountBelong;	//账号所属, 0-运维系统账号，1-业务账号
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String accountSource; // 账号来源

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date lastLoginTime; // 账号上一次登录时间

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String lastLoginIp; // 账号上一次登录IP
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private Integer loginFail; 	// 账号登录失败次数，超过阈值，冻结，阈值来源于配置中心的 integrated.security.login-fail.fail-num 属性

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date validStartTime; // 账号有效期的开始时间

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date validEndTime;  // 账号有效期的结束时间
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String validLast;	//账号有效期，0-期内有效，1-永久有效

	@TableField(fill = FieldFill.INSERT, updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date createTime; 	// 创建时间

	@TableField(fill = FieldFill.INSERT_UPDATE, updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date updateTime; 	// 更新时间

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String updateUser; // 最后一次操作者ID

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String remark; // 备注

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private Integer valid; // 0-不可用，1-可用

	@TableField(exist = false)
	private String groupName; 		// 集团名称
	
	@TableField(exist = false)
	private String type;	//类型 create-创建，update-更新
	
	@TableField(exist = false)
	private List<SaasGroup> groups;	//集团信息集合
	
	@TableField(exist = false)
	private int projectNum;		//项目个数

	@TableField(exist = false)
	private List<SaasRole> roles;	//角色集合
	
	@TableField(exist = false)
	private List<SaasMenu> auths;	//菜单权限树集合
	
	@TableField(exist = false)
	private Set<String> accountIds;
	
	@TableField(exist = false)
	private String casType; 	// 关联类型，0-主岗，1-副岗，2-业务超管默认角色
	
	public static class Builder {

		private LambdaQueryWrapper<SaasAccount> queryWrapper = null;

		private LambdaUpdateWrapper<SaasAccount> updateWrapper = null;

		public Builder createQueryWrapper() {
			queryWrapper = new LambdaQueryWrapper<SaasAccount>();
			return this;
		}

		public Builder createUpdateWrapper() {
			updateWrapper = new LambdaUpdateWrapper<SaasAccount>();
			return this;
		}

		public Builder idEq(String id) {
			if (StringUtil.isNotBlank(id)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getId, id);
				} else {
					queryWrapper.eq(SaasAccount::getId, id);
				}
			}
			return this;
		}
		
		public Builder idIn(Set<String> ids) {
			if (CollectionUtil.isNotEmpty(ids)) {
				if (updateWrapper != null) {
					updateWrapper.in(SaasAccount::getId, ids);
				} else {
					queryWrapper.in(SaasAccount::getId, ids);
				}
			}
			return this;
		}

		public Builder groupCodeEq(String groupCode) {
			if (StringUtil.isNotBlank(groupCode)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getGroupCode, groupCode);
				} else {
					queryWrapper.eq(SaasAccount::getGroupCode, groupCode);
				}
			}
			return this;
		}

		public Builder terminalLike(String appId) {
			if (StringUtil.isNotBlank(appId)) {
				if (updateWrapper != null) {
					updateWrapper.apply("TERMINAL LIKE CONCAT('%', " + appId + ",'%')");
				} else {
					queryWrapper.apply("TERMINAL LIKE CONCAT('%', " + appId + ",'%')");
				}
			}
			return this;
		}
		
		public Builder usernameEq(String username) {
			if (StringUtil.isNotBlank(username)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getUsername, username);
				} else {
					queryWrapper.eq(SaasAccount::getUsername, username);
				}
			}
			return this;
		}

		public Builder usernameLike(String username) {
			if (StringUtil.isNotBlank(username)) {
				if (updateWrapper != null) {
					updateWrapper.like(SaasAccount::getUsername, username);
				} else {
					queryWrapper.like(SaasAccount::getUsername, username);
				}
			}
			return this;
		}

		public Builder passwordEq(String password) {
			if (StringUtil.isNotBlank(password)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getPassword, password);
				} else {
					queryWrapper.eq(SaasAccount::getPassword, password);
				}
			}
			return this;
		}

		public Builder phoneNumEq(String phoneNum) {
			if (StringUtil.isNotBlank(phoneNum)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getPhoneNum, phoneNum);
				} else {
					queryWrapper.eq(SaasAccount::getPhoneNum, phoneNum);
				}
			}
			return this;
		}

		public Builder mailEq(String mail) {
			if (StringUtil.isNotBlank(mail)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getMail, mail);
				} else {
					queryWrapper.eq(SaasAccount::getMail, mail);
				}
			}
			return this;
		}

		public Builder headPortraitEq(String headPortrait) {
			if (StringUtil.isNotBlank(headPortrait)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getHeadPortrait, headPortrait);
				} else {
					queryWrapper.eq(SaasAccount::getHeadPortrait, headPortrait);
				}
			}
			return this;
		}

		public Builder accountTypeEq(String accountType) {
			if (StringUtil.isNotBlank(accountType)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getAccountType, accountType);
				} else {
					queryWrapper.eq(SaasAccount::getAccountType, accountType);
				}
			}
			return this;
		}
		
		public Builder accountSourceEq(String accountSource) {
			if (StringUtil.isNotBlank(accountSource)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getAccountSource, accountSource);
				} else {
					queryWrapper.eq(SaasAccount::getAccountSource, accountSource);
				}
			}
			return this;
		}
		
		public Builder accountBelongEq(String accountBelong) {
			if (StringUtil.isNotBlank(accountBelong)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getAccountBelong, accountBelong);
				} else {
					queryWrapper.eq(SaasAccount::getAccountBelong, accountBelong);
				}
			}
			return this;
		}

		public Builder lastLoginTimeEq(Date lastLoginTime) {
			if (null != lastLoginTime) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getLastLoginTime, lastLoginTime);
				} else {
					queryWrapper.eq(SaasAccount::getLastLoginTime, lastLoginTime);
				}
			}
			return this;
		}

		public Builder lastLoginIpEq(String lastLoginIp) {
			if (StringUtil.isNotBlank(lastLoginIp)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getLastLoginIp, lastLoginIp);
				} else {
					queryWrapper.eq(SaasAccount::getLastLoginIp, lastLoginIp);
				}
			}
			return this;
		}

		public Builder validStartTimeEq(Date validStartTime) {
			if (null != validStartTime) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getValidStartTime, validStartTime);
				} else {
					queryWrapper.eq(SaasAccount::getValidStartTime, validStartTime);
				}
			}
			return this;
		}

		public Builder validEndTimeEq(Date validEndTime) {
			if (null != validEndTime) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getValidEndTime, validEndTime);
				} else {
					queryWrapper.eq(SaasAccount::getValidEndTime, validEndTime);
				}
			}
			return this;
		}
		
		/**
		 * 当前时间是否在有效期内
		 * @param validEndTime
		 * @return
		 */
		public Builder validDateBetween(Date currentDate) {
			if (null != currentDate) {
				if (updateWrapper != null) {
					updateWrapper.le(SaasAccount::getValidStartTime, currentDate);
					updateWrapper.ge(SaasAccount::getValidEndTime, currentDate);
				} else {
					queryWrapper.le(SaasAccount::getValidStartTime, currentDate);
					queryWrapper.ge(SaasAccount::getValidEndTime, currentDate);
				}
			}
			return this;
		}

		public Builder validLastEq(String validLast) {
			if (StringUtil.isNotBlank(validLast)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getValidLast, validLast);
				} else {
					queryWrapper.eq(SaasAccount::getValidLast, validLast);
				}
			}
			return this;
		}
		
		public Builder createTimeEq(Date createTime) {
			if (null != createTime) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getCreateTime, createTime);
				} else {
					queryWrapper.eq(SaasAccount::getCreateTime, createTime);
				}
			}
			return this;
		}

		public Builder updateTimeEq(Date updateTime) {
			if (null != updateTime) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getUpdateTime, updateTime);
				} else {
					queryWrapper.eq(SaasAccount::getUpdateTime, updateTime);
				}
			}
			return this;
		}

		public Builder updateUserEq(String updateUser) {
			if (StringUtil.isNotBlank(updateUser)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getUpdateUser, updateUser);
				} else {
					queryWrapper.eq(SaasAccount::getUpdateUser, updateUser);
				}
			}
			return this;
		}

		public Builder remarkEq(String remark) {
			if (StringUtil.isNotBlank(remark)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getRemark, remark);
				} else {
					queryWrapper.eq(SaasAccount::getRemark, remark);
				}
			}
			return this;
		}

		public Builder validEq(Integer valid) {
			if (null != valid) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccount::getValid, valid);
				} else {
					queryWrapper.eq(SaasAccount::getValid, valid);
				}
			}
			return this;
		}

		public LambdaQueryWrapper<SaasAccount> builderQueryWrapper() {
			return queryWrapper;
		}

		public LambdaUpdateWrapper<SaasAccount> builderUpdateWrapper() {
			return updateWrapper;
		}
	}
}
