package com.persagy.account.service.impl;

import java.util.List;
import java.util.Set;

import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.persagy.account.mapper.SaasAccountMapper;
import com.persagy.account.pojo.dto.SaasAccount;
import com.persagy.account.service.ISaasAccountService;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.service.impl.SuperServiceImpl;
import com.persagy.common.utils.StringUtil;

import cn.hutool.core.collection.CollectionUtil;

/**
 * 账号信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Service
public class SaasAccountServiceImpl extends SuperServiceImpl<SaasAccountMapper, SaasAccount> implements ISaasAccountService {

	@Override
	public SaasAccount getOne(String id, String groupCode, String username) {
		LambdaQueryWrapper<SaasAccount> queryWrapper = new SaasAccount.Builder().createQueryWrapper()
				.idEq(id).groupCodeEq(groupCode).usernameEq(username)
				.validEq(SaasCommonConstant.STATUS_1).builderQueryWrapper();
		queryWrapper.select(SaasAccount::getId, SaasAccount::getGroupCode, SaasAccount::getTerminal, SaasAccount::getUsername,
    			SaasAccount::getPhoneNum, SaasAccount::getMail, SaasAccount::getHeadPortrait, SaasAccount::getAccountBelong, 
    			SaasAccount::getAccountSource, SaasAccount::getAccountType, SaasAccount::getLastLoginIp, SaasAccount::getLastLoginTime,
    			SaasAccount::getValidStartTime, SaasAccount::getValidEndTime, SaasAccount::getValidLast, SaasAccount::getCreateTime,
    			SaasAccount::getUpdateTime, SaasAccount::getUpdateUser, SaasAccount::getRemark, SaasAccount::getValid);
		return this.baseMapper.selectOne(queryWrapper);
	}
	
	@Override
	public List<SaasAccount> queryRoleAccountList(SaasAccount saasAccount, String roleId) {
		return this.baseMapper.queryRoleAccountList(saasAccount, Sets.newHashSet(roleId), null, null);
	}
	
	@Override
	public Page<SaasAccount> queryRoleAccountPage(SaasAccount saasAccount, Set<String> roleIds, Integer page, Integer size) {
		Page<SaasAccount> pageRecord = new Page<SaasAccount>();
		int startRow = (page - 1) * size;
    	pageRecord.setCurrent(page);
		pageRecord.setSize(size);
		List<SaasAccount> list = this.baseMapper.queryRoleAccountList(saasAccount, roleIds, startRow, size);
		if (CollectionUtil.isEmpty(list)) {
			pageRecord.setRecords(Lists.newArrayList());
			pageRecord.setTotal(0);
			return pageRecord;
		}
		// 查询出总记录数大小
		int count = this.baseMapper.countRoleAccountList(saasAccount, roleIds);
		pageRecord.setRecords(list);
		pageRecord.setTotal(count);
		return pageRecord;
	}
	
	@Override
	public Page<SaasAccount> queryAccountPageList(SaasAccount saasAccount, int page, int size) {
		Page<SaasAccount> pageRecord = new Page<SaasAccount>();
		int startRow = (page - 1) * size;
    	pageRecord.setCurrent(page);
		pageRecord.setSize(size);
		List<SaasAccount> list = this.baseMapper.queryAccountGroupList(saasAccount, startRow, size);
		if (CollectionUtil.isEmpty(list)) {
			pageRecord.setRecords(Lists.newArrayList());
			pageRecord.setTotal(0);
			return pageRecord;
		}
		// 查询出总记录数大小
		LambdaQueryWrapper<SaasAccount> queryWrapper = new SaasAccount.Builder().createQueryWrapper()
    			.idEq(saasAccount.getId()).idIn(saasAccount.getAccountIds()).updateUserEq(saasAccount.getUpdateUser())
    			.groupCodeEq(saasAccount.getGroupCode()).usernameLike(saasAccount.getUsername()).accountTypeEq(saasAccount.getAccountType())
    			.accountSourceEq(saasAccount.getAccountSource()).phoneNumEq(saasAccount.getPhoneNum()).mailEq(saasAccount.getMail())
    			.accountBelongEq(saasAccount.getAccountBelong()).validEq(saasAccount.getValid()).builderQueryWrapper();
		queryWrapper.select(SaasAccount::getId);
		int count = this.baseMapper.selectCount(queryWrapper);
		pageRecord.setRecords(list);
		pageRecord.setTotal(count);
		return pageRecord;
	}
	
	@Override
	public LambdaQueryWrapper<SaasAccount> getQueryWrapper(SaasAccount saasAccount) {
		LambdaQueryWrapper<SaasAccount> queryWrapper = new SaasAccount.Builder().createQueryWrapper()
    			.idEq(saasAccount.getId()).idIn(saasAccount.getAccountIds()).updateUserEq(saasAccount.getUpdateUser())
    			.groupCodeEq(saasAccount.getGroupCode()).usernameLike(saasAccount.getUsername()).accountTypeEq(saasAccount.getAccountType())
    			.accountSourceEq(saasAccount.getAccountSource()).phoneNumEq(saasAccount.getPhoneNum()).mailEq(saasAccount.getMail())
    			.accountBelongEq(saasAccount.getAccountBelong()).validEq(saasAccount.getValid()).builderQueryWrapper();
    	queryWrapper.select(SaasAccount::getId, SaasAccount::getGroupCode, SaasAccount::getTerminal, SaasAccount::getUsername,
    			SaasAccount::getPhoneNum, SaasAccount::getMail, SaasAccount::getHeadPortrait, SaasAccount::getAccountBelong, 
    			SaasAccount::getAccountSource, SaasAccount::getAccountType, SaasAccount::getLastLoginIp, SaasAccount::getLastLoginTime,
    			SaasAccount::getValidStartTime, SaasAccount::getValidEndTime, SaasAccount::getValidLast, SaasAccount::getCreateTime,
    			SaasAccount::getUpdateTime, SaasAccount::getUpdateUser, SaasAccount::getRemark, SaasAccount::getValid);
		return queryWrapper;
	}

	@Override
	public boolean validAccountName(String username, String mail, String phone, String id, boolean isUpdate) {
		LambdaQueryWrapper<SaasAccount> queryWrapper = null;
		SaasAccount selectOne = null;
		if (StringUtil.isNotBlank(username)) {
			queryWrapper = new SaasAccount.Builder().createQueryWrapper().usernameEq(username).builderQueryWrapper();
			selectOne = this.baseMapper.selectOne(queryWrapper);
			if (selectOne != null && !isUpdate) {
				return false;
			}
			if (isUpdate && selectOne != null && !selectOne.getId().equals(id)) {
				return false;
			}
		}
		if (StringUtil.isNotBlank(mail)) {
			queryWrapper = new SaasAccount.Builder().createQueryWrapper().mailEq(mail).builderQueryWrapper();
			selectOne = this.baseMapper.selectOne(queryWrapper);
			if (selectOne != null && !isUpdate) {
				return false;
			}
			if (isUpdate && selectOne != null && !selectOne.getId().equals(id)) {
				return false;
			}
		}
		if (StringUtil.isNotBlank(phone)) {
			queryWrapper = new SaasAccount.Builder().createQueryWrapper().phoneNumEq(phone).builderQueryWrapper();
			selectOne = this.baseMapper.selectOne(queryWrapper);
			if (selectOne != null && !isUpdate) {
				return false;
			}
			if (isUpdate && selectOne != null && !selectOne.getId().equals(id)) {
				return false;
			}
		}
		return true;
	}



    @Override
    public LambdaQueryWrapper<SaasAccount> getQueryWrapperByUsername(SaasAccount saasAccount) {
        LambdaQueryWrapper<SaasAccount> queryWrapper = new SaasAccount.Builder().createQueryWrapper()
                .groupCodeEq(saasAccount.getGroupCode()).usernameEq(saasAccount.getUsername()).builderQueryWrapper();
        queryWrapper.select(SaasAccount::getId, SaasAccount::getGroupCode, SaasAccount::getTerminal, SaasAccount::getUsername,
                SaasAccount::getPhoneNum, SaasAccount::getMail, SaasAccount::getHeadPortrait, SaasAccount::getAccountBelong,
                SaasAccount::getAccountSource, SaasAccount::getAccountType, SaasAccount::getLastLoginIp, SaasAccount::getLastLoginTime,
                SaasAccount::getValidStartTime, SaasAccount::getValidEndTime, SaasAccount::getValidLast, SaasAccount::getCreateTime,
                SaasAccount::getUpdateTime, SaasAccount::getUpdateUser, SaasAccount::getRemark, SaasAccount::getValid);
        return queryWrapper;
    }

}
