package com.persagy.account.service.impl;

import java.util.List;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.persagy.account.mapper.SaasProductPageMapper;
import com.persagy.account.pojo.dto.SaasProductPage;
import com.persagy.account.service.ISaasProductPageService;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.service.impl.SuperServiceImpl;
import com.persagy.common.utils.StringUtil;

/**
 * 产品页面配置
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-05-06 10:12:47
 */
@Service
public class SaasProductPageServiceImpl extends SuperServiceImpl<SaasProductPageMapper, SaasProductPage> implements ISaasProductPageService {

	@Override
	public List<SaasProductPage> queryGroupProductPages(String groupCode, String productId, String angle) {
		return this.baseMapper.queryGroupProductPages(groupCode, productId, angle);
	}

	@Override
	public List<SaasProductPage> queryProjectProductPages(String groupCode, String projectId, String productId, String angle) {
		return this.baseMapper.queryProjectProductPages(groupCode, projectId, productId, angle);
	}

	@Override
	public List<SaasProductPage> queryAccountProductPages(String accountId, String productId, String menuId, String parentId, String angle) {
		return this.baseMapper.queryAccountProductPages(accountId, productId, menuId, parentId, angle);
	}

	@Override
	@Transactional
	public boolean createSaasProductPage(SaasProductPage saasProductPage) {
		// 判断是否是落地页，落地页的时候，取消其余的落地页设置
		this.updatePageFloor(saasProductPage.getProductId(), saasProductPage.getPageFloor());
		Integer result = this.baseMapper.insert(saasProductPage);
		return SqlHelper.retBool(result);
	}

	@Override
	@Transactional
	public boolean updateSaasProductPage(SaasProductPage saasProductPage) {
		// 判断是否是落地页，落地页的时候，取消其余的落地页设置
		this.updatePageFloor(saasProductPage.getProductId(), saasProductPage.getPageFloor());
		Integer result = this.baseMapper.updateById(saasProductPage);
		return SqlHelper.retBool(result);
	}
	
	@Override
	public boolean validPageName(String productId, String pageName, String pageId) {
		LambdaQueryWrapper<SaasProductPage> queryWrapper = new SaasProductPage.BuilderQueryWrapper().productIdEq(productId).pageNameEq(pageName).builder();
		SaasProductPage selectOne = this.baseMapper.selectOne(queryWrapper);
		if (selectOne == null) {
			return true;
		}
		if (StringUtil.isNotBlank(pageId) && selectOne.getId().equals(pageId)) {
			return true;
		}
		return false;
	}

	/**
	 * 更新落地页选项
	 * @param productId
	 * @param pageFloor
	 */
	private void updatePageFloor(String productId, String pageFloor) {
		if (SaasCommonConstant.STR_STATUS_0.equals(pageFloor)) {
			return;
		}
		String pageFloorSql = null;
		if (SaasCommonConstant.STR_STATUS_1.equals(pageFloor)) {
			pageFloorSql = "PAGE_FLOOR = (CASE PAGE_FLOOR WHEN '3' THEN '2' WHEN '1' THEN '0' ELSE PAGE_FLOOR END)";
		} else if (SaasCommonConstant.STR_STATUS_2.equals(pageFloor)) {
			pageFloorSql = "PAGE_FLOOR = (CASE PAGE_FLOOR WHEN '3' THEN '1' WHEN '2' THEN '0' ELSE PAGE_FLOOR END)";
		} else {
			pageFloorSql = "PAGE_FLOOR = '0'";
		}
		LambdaUpdateWrapper<SaasProductPage> updateWrapper = new SaasProductPage.BuilderUpdateWrapper().productIdEq(productId).builder();
		updateWrapper.setSql(pageFloorSql);
		this.baseMapper.update(null, updateWrapper);
	}
	
}
